# -*- coding: utf-8 -*-
"""

    Инструменты обработки JSON файлов

"""
# встроенные модули
import os
import sys
import json
from typing import Dict, Any, Optional

# сторонние модули
from loguru import logger


def json_config_load(filename: str, config_name: str,
                     default_config: Optional[str]) -> Dict[str, Any]:
    """
    Открыть указанный файл и прочитать его содержимое
    """
    path = os.path.join(os.getcwd(), filename)
    try:
        with open(path, mode='r', encoding='utf-8') as file:
            data = json.load(file)

            if config_name in data:
                config_data = data[config_name]

            elif default_config in data:
                config_data = data[default_config]

            else:
                raise KeyError

            return config_data

    except FileNotFoundError:
        logger.critical(f'Не найден файл конфигурации: "{path}"')

    except KeyError:
        logger.critical(f'Не удалось загрузить конфигурацию "{config_name}" из файла: "{path}"')

    sys.exit(1)



# -*- coding: utf-8 -*-
"""

    Простые инструменты для работы с JSON-RPC

"""
# встроенные модули
from typing import Dict, Callable, Optional, Any


def method(container: Dict[str, Callable]) -> Callable:
    """
    Декоратор регистрации методов в JSON-RPC API
    """
    def wrapper(func: Callable) -> Callable:
        """
        Объёртка для создания замыкания для передачи container
        """
        container[func.__name__] = func
        return func
    return wrapper


def form_request(method_name: str, request_id: Optional[int] = None, **kwargs) -> Dict[str, Any]:
    """
    Собрать запрос к API
    """
    request = {
        "jsonrpc": "2.0",
        "method": method_name,
        "params": {**kwargs},
        "id": request_id
    }
    return request


# TODO - deprecate this
def form_valid_response(result: Any, request_id: Optional[int] = None) -> Dict[str, Any]:
    """
    Собрать ответ из API
    """
    response = {"jsonrpc": "2.0", "result": result, "id": request_id}
    return response


def result(output: Any, request_id: Optional[int] = None) -> Dict[str, Any]:
    """
    Успешный результат
    """
    return {"jsonrpc": "2.0", "result": output, "id": request_id}


def error(output: str, request_id: Optional[int] = None) -> Dict[str, Any]:
    """
    Неудачный результат
    """
    return {"jsonrpc": "2.0", "error": output, "id": request_id}



# -*- coding: utf-8 -*-
"""

    Базовый класс для управления конфигурациями

"""
# встроенные модули
import sys
from typing import Optional

# модули проекта
from trivial_tools.files.json import json_config_load
from trivial_tools.system.envs import get_env_variable
from trivial_tools.classes.singleton_base import Singleton


class BaseConfig(metaclass=Singleton):
    """
    Базовый класс для хранения параметров
    """
    def __init__(self, **kwargs):
        """
        Автоматическая инициация
        """
        for key, value in kwargs.items():
            setattr(self, key, value)

    def to_dict(self) -> dict:
        """
        Преобразовать в словарь
        """
        result = {key: value for key, value in self.__dict__.items()}
        return result

    def alter(self, **kwargs):
        """
        Получить аналогичный конфиг, но с другими параметрами
        """
        values = {**self.to_dict(), **kwargs}
        result = type(self)(**values)
        return result

    @classmethod
    def from_json(cls, filename: str, config_name: str, default_config: Optional[str] = None):
        """
        Загрузить настройки из json файла
        """
        parameters = json_config_load(filename, config_name, default_config)
        instance = cls(**parameters)
        return instance

    def __str__(self):
        """
        Текстовое представление
        """
        name = type(self).__name__

        longest = 1
        fields = []

        for field_name in vars(self):
            value = getattr(self, field_name)
            fields.append((field_name, value))
            longest = max(longest, len(field_name))

        result = [f'{name}(']
        for i, (field_name, value) in enumerate(fields, start=1):
            if isinstance(value, (list, tuple)) and value:
                result.append(
                    ('\t{0:02d}. {1:' + str(longest) + '} = [').format(i, field_name)
                )
                for sub_element in value:
                    result.append(f'\t        ' + ' ' * longest * 1 + f'{sub_element},')
                result.append(f'\t       ' + ' ' * longest * 1 + ']')
            else:
                result.append(
                    ('\t{0:02d}. {1:' + str(longest) + '} = {2}').format(i, field_name, value)
                )
        result.append(')')
        return '\n'.join(result)

    def __repr__(self):
        """
        Текстовое представление
        """
        name = type(self).__name__
        result = []
        for field_name in vars(self):
            value = getattr(self, field_name)
            result.append(f'{field_name}={value!r}')
        return f'{name}(' + ', '.join(result) + ')'

    @classmethod
    def get_instance(cls):
        """
        Получить экземпляр
        """
        instance = Singleton.get_instance(cls)
        return instance

    @classmethod
    def get_config(cls, env_name: str, config_filename: str = 'config.json'):
        """
        Получить экземпляр конфига
        """
        config = cls.from_json(
            filename=config_filename,
            config_name=cls.get_config_name(),
            default_config=get_env_variable(env_name)
        )
        return config

    @staticmethod
    def get_config_name() -> Optional[str]:
        """
        Примитивный способ получить имя конфигурации из аргументов запуска
        """
        config_name = None
        if len(sys.argv) >= 3 and sys.argv[1] == '--config':
            config_name = sys.argv[2]
        return config_name
